/* 
 For 12 DC motors on 3 stacked motor shields controlled via serial communication with Max/MSP
 For use with the Adafruit Motor Shield v2 http://www.adafruit.com/products/1438
 
 Code and documentation:
 http://liveelectronics.musinou.net/DCMotorsSerialControl.php

 ####################################################
 # 2014-2015 by http://liveelectronics.musinou.net/ #
 ####################################################
 # Initially developed for Réseau's Sphéro-automaton V2 http://musinou.net/note/Sph%C3%A9ro-automaton
 # Initial development partly funded by http://www.oboro.net/ where Sphéro-automaton V2 was first used
 #
 # Documentation and sharable Max/MSP patch development funded by le Cirque du Soleil
 #
 */


#include <Wire.h>
#include <Adafruit_MotorShield.h>
#include "utility/Adafruit_PWMServoDriver.h"

// Create the motor shield object (no modification in the address-select pins)
Adafruit_MotorShield MotorShield_60(0x60);
// Create the motor shield object (need to modify the the address-select pins) https://learn.adafruit.com/adafruit-motor-shield-v2-for-arduino/stacking-shields
Adafruit_MotorShield MotorShield_61(0x61);
Adafruit_MotorShield MotorShield_62(0x62);

// Select 'ports'
//1st shield
Adafruit_DCMotor *a = MotorShield_60.getMotor(1);
Adafruit_DCMotor *b = MotorShield_60.getMotor(2);
Adafruit_DCMotor *c = MotorShield_60.getMotor(3);
Adafruit_DCMotor *d = MotorShield_60.getMotor(4);
//other shields
Adafruit_DCMotor *e = MotorShield_61.getMotor(1);
Adafruit_DCMotor *f = MotorShield_61.getMotor(2);
Adafruit_DCMotor *g = MotorShield_61.getMotor(3);
Adafruit_DCMotor *h = MotorShield_61.getMotor(4);

Adafruit_DCMotor *i = MotorShield_62.getMotor(1);
Adafruit_DCMotor *j = MotorShield_62.getMotor(2);
Adafruit_DCMotor *k = MotorShield_62.getMotor(3);
Adafruit_DCMotor *l = MotorShield_62.getMotor(4);

//led pin for running the test
int led = 13;


void setup() {

  //for serial communication with Max/MSP  
  Serial.begin(38400);

  // create with the default frequency 1.6KHz
  MotorShield_60.begin();
  MotorShield_61.begin();
  MotorShield_62.begin();

  pinMode(led, OUTPUT); 

  //for testing motors
  int motorOnFor= 800;
  int delayBetweenMotorTests= 400;
  int ledDuration= 130;

  //run first test; each motor one by one
  boolean run = false; //set to true if you want this test to run at startup;
  if (run == true)
  {
    flash(led, 1, ledDuration, ledDuration);
    testRun(a, motorOnFor, delayBetweenMotorTests);

    flash(led, 2, ledDuration, ledDuration);
    testRun(b, motorOnFor, delayBetweenMotorTests);

    flash(led, 3, ledDuration, ledDuration);
    testRun(c, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration);
    testRun(d, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 1, ledDuration, ledDuration);
    testRun(e, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 2, ledDuration, ledDuration);
    testRun(f, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 3, ledDuration, ledDuration);
    testRun(g, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration);
    testRun(h, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 1, ledDuration, ledDuration);
    testRun(i, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 2, ledDuration, ledDuration);
    testRun(j, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 3, ledDuration, ledDuration);
    testRun(k, motorOnFor, delayBetweenMotorTests);

    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration); 
    delay (ledDuration); 
    flash(led, 4, ledDuration, ledDuration);
    testRun(l, motorOnFor, delayBetweenMotorTests);
  }

  //run second test, all motors run at the same time
  run = false;  //set to true if you want this test to run at startup;
  if (run == true)
  { 
    int valueFromMax = 255;

    motorAction(a, valueFromMax);
    motorAction(b, valueFromMax);
    motorAction(c, valueFromMax);
    motorAction(d, valueFromMax);
    motorAction(e, valueFromMax);
    motorAction(f, valueFromMax);
    motorAction(g, valueFromMax);
    motorAction(h, valueFromMax);
    motorAction(i, valueFromMax);
    motorAction(j, valueFromMax);
    motorAction(k, valueFromMax);
    motorAction(l, valueFromMax);

    //runs for 2sec
    delay (2000);

    motorAction(a, 0);
    motorAction(b, 0);
    motorAction(c, 0);
    motorAction(d, 0);
    motorAction(e, 0);
    motorAction(f, 0);
    motorAction(g, 0);
    motorAction(h, 0);
    motorAction(i, 0);
    motorAction(j, 0);
    motorAction(k, 0);
    motorAction(l, 0);
  }

}

void loop() 
{

  // if there's any serial available, read it:
  while (Serial.available() > 0) 
  {

    int controlledOuput=Serial.read(); // read first char
    //Serial.print("controlledOuput : "); Serial.println(controlledOuput);

    if(controlledOuput >= 'A' && controlledOuput <= 'z')// controlledOuput most be a letter
    {
      int valueFromMax = Serial.parseInt();
      //Serial.print("valueFromMax : "); Serial.println(valueFromMax);

      //uses controlledOuput to action the right motor
      switch(controlledOuput)
      {

      case 'a':
        motorAction(a, valueFromMax); 
        break;
      case 'b': 
        motorAction(b, valueFromMax); 
        break;
      case 'c': 
        motorAction(c, valueFromMax); 
        break;
      case 'd': 
        motorAction(d, valueFromMax); 
        break;
      case 'e': 
        motorAction(e, valueFromMax); 
        break;
      case 'f': 
        motorAction(f, valueFromMax); 
        break;    
      case 'g': 
        motorAction(g, valueFromMax); 
        break;
      case 'h': 
        motorAction(h, valueFromMax); 
        break;
      case 'i': 
        motorAction(i, valueFromMax); 
        break;     
      case 'j': 
        motorAction(j, valueFromMax); 
        break;
      case 'k': 
        motorAction(k, valueFromMax); 
        break;
      case 'l': 
        motorAction(l, valueFromMax); 
        break;

      case 'Z':
        if(valueFromMax > 0) 
        {
          digitalWrite(led, LOW);
        } 
        else 
        {
          digitalWrite(led, HIGH);
        }   
        break;   
      }
    }
  }
}

//value from max contrain between -255 and 255, if positive motor goes forward, if negative motor goes backward, if nul release motor
void motorAction (Adafruit_DCMotor *motor, int valueFromMax)
{
  valueFromMax = constrain(valueFromMax, -255, 255);
  if(valueFromMax > 0){
    motor->run(FORWARD); 
    motor->setSpeed(valueFromMax);
  } 
  else if(valueFromMax < 0){
    motor->run(BACKWARD); 
    motor->setSpeed(abs(valueFromMax));
  }
  else {
    motor->run(RELEASE);
  }      
}

//runs motor forward for a time and delay
void testRun (Adafruit_DCMotor *motor, int motorOnFor, int delayBetweenMotorTests)
{
  motor->run(FORWARD);
  motor->setSpeed(255);
  delay (motorOnFor);
  motor->run(RELEASE);
  delay (delayBetweenMotorTests);
}

//flashes a LED and delay
void flash (int whatLed, int ammountOfRepetition, int howLongOn, int howLongOff)
{
  for (int i=1; i <= ammountOfRepetition; i++)
  {
    digitalWrite(whatLed, LOW);
    delay(howLongOn);
    digitalWrite(whatLed, HIGH);
    delay(howLongOff);
  }
}
